@@ -1,7 +1,9 @@
 #ifndef _FDEVENT_H_
 #define _FDEVENT_H_
 
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif
 #include "settings.h"
 #include "bitset.h"
 
@@ -35,6 +37,9 @@
 #  include <winsock2.h>
 # endif
 # define USE_SELECT
+# ifdef HAVE_SYS_SELECT_H
+#  include <sys/select.h>
+# endif
 #endif
 
 #if defined HAVE_SYS_DEVPOLL_H && defined(__sun)
@@ -103,18 +108,14 @@ typedef struct {
  * 
  */
 
-typedef struct fdnode {
+typedef struct _fdnode {
 	fdevent_handler handler;
 	void *ctx;
 	int fd;
 	
-	struct fdnode *prev, *next;
+	struct _fdnode *prev, *next;
 } fdnode;
 
-typedef struct {
-	fdnode *first, *last;
-} fdlist;
-
 typedef struct {
 	int *ptr;
 	
@@ -129,7 +130,6 @@ typedef struct {
 typedef struct fdevents {
 	fdevent_handler_t type;
 	
-	fdlist fdlist;
 	fdnode **fdarray;
 	size_t maxfds;
 	
