@@ -15,7 +15,6 @@
 #ifdef USE_OPENSSL
 # include <openssl/md5.h>
 #else
-# include "md5_global.h"
 # include "md5.h"
 #endif
 
@@ -38,7 +37,7 @@ typedef struct {
 	buffer *secret;
 	buffer *uri_prefix;
 	
-	time_t timeout;
+	unsigned short timeout;
 } plugin_config;
 
 typedef struct {
