@@ -168,7 +168,7 @@ handler_t http_response_prepare(server *srv, connection *con) {
 		 * - path
 		 * - query
 		 * 
-		 * (scheme)://(authority)(path)?(query)
+		 * (scheme)://(authority)(path)?(query)#fragment
 		 * 
 		 * 
 		 */
@@ -182,7 +182,13 @@ handler_t http_response_prepare(server *srv, connection *con) {
 		config_patch_connection(srv, con, COMP_HTTP_REFERER);   /* Referer:     */
 		config_patch_connection(srv, con, COMP_HTTP_USERAGENT); /* User-Agent:  */
 		config_patch_connection(srv, con, COMP_HTTP_COOKIE);    /* Cookie:  */
-		
+	
+		/** their might be a fragment which has to be cut away */
+		if (NULL != (qstr = strchr(con->request.uri->ptr, '#'))) {
+			con->request.uri->used = qstr - con->request.uri->ptr;
+			con->request.uri->ptr[con->request.uri->used++] = '\0';
+		}
+
 		/** extract query string from request.uri */
 		if (NULL != (qstr = strchr(con->request.uri->ptr, '?'))) {
 			buffer_copy_string    (con->uri.query, qstr + 1);
@@ -261,7 +267,13 @@ handler_t http_response_prepare(server *srv, connection *con) {
 		 */
 		
 		config_patch_connection(srv, con, COMP_HTTP_URL); /* HTTPurl */
-		
+		config_patch_connection(srv, con, COMP_HTTP_QUERYSTRING); /* HTTPqs */
+
+		/* do we have to downgrade to 1.0 ? */
+		if (!con->conf.allow_http11) {
+			con->request.http_version = HTTP_VERSION_1_0;
+		}
+
 		switch(r = plugins_call_handle_uri_clean(srv, con)) {
 		case HANDLER_GO_ON:
 			break;
@@ -451,7 +463,7 @@ handler_t http_response_prepare(server *srv, connection *con) {
 			log_error_write(srv, __FILE__, __LINE__,  "s",  "-- handling physical path");
 			log_error_write(srv, __FILE__, __LINE__,  "sb", "Path         :", con->physical.path);
 		}
-		
+
 		if (HANDLER_ERROR != stat_cache_get_entry(srv, con, con->physical.path, &sce)) {
 			/* file exists */
 			
@@ -459,7 +471,19 @@ handler_t http_response_prepare(server *srv, connection *con) {
 				log_error_write(srv, __FILE__, __LINE__,  "s",  "-- file found");
 				log_error_write(srv, __FILE__, __LINE__,  "sb", "Path         :", con->physical.path);
 			}
-			
+#ifdef HAVE_LSTAT
+			if ((sce->is_symlink != 0) && !con->conf.follow_symlink) {
+				con->http_status = 403;
+
+				if (con->conf.log_request_handling) {
+					log_error_write(srv, __FILE__, __LINE__,  "s",  "-- access denied due symlink restriction");
+					log_error_write(srv, __FILE__, __LINE__,  "sb", "Path         :", con->physical.path);
+				}
+
+				buffer_reset(con->physical.path);
+				return HANDLER_FINISHED;
+			};
+#endif
 			if (S_ISDIR(sce->st.st_mode)) {
 				if (con->physical.path->ptr[con->physical.path->used - 2] != '/') {
 					/* redirect to .../ */
@@ -468,7 +492,11 @@ handler_t http_response_prepare(server *srv, connection *con) {
 					
 					return HANDLER_FINISHED;
 				}
+#ifdef HAVE_LSTAT
+			} else if (!S_ISREG(sce->st.st_mode) && !sce->is_symlink) {
+#else
 			} else if (!S_ISREG(sce->st.st_mode)) {
+#endif
 				/* any special handling of non-reg files ?*/
 
 
@@ -541,7 +569,7 @@ handler_t http_response_prepare(server *srv, connection *con) {
 				}
 				
 				if (slash) pathinfo = slash;
-			} while ((found == 0) && (slash != NULL) && (slash - srv->tmp_buf->ptr > con->physical.basedir->used - 2));
+			} while ((found == 0) && (slash != NULL) && ((size_t)(slash - srv->tmp_buf->ptr) > (con->physical.basedir->used - 2)));
 			
 			if (found == 0) {
 				/* no it really doesn't exists */
