@@ -87,14 +87,15 @@ sub start_proc {
 	# pre-process configfile if necessary
 	#
 
-	unlink($self->{TESTDIR}."/tmp/cfg.file");
-	system("cat ".$self->{SRCDIR}."/".$self->{CONFIGFILE}.' | perl -pe "s#\@SRCDIR\@#'.$self->{BASEDIR}.'/tests/#" > '.$self->{TESTDIR}.'/tmp/cfg.file');
+	$ENV{'SRCDIR'} = $self->{BASEDIR}.'/tests';
 
 	unlink($self->{LIGHTTPD_PIDFILE});
-	if (1) {
-		system($self->{LIGHTTPD_PATH}." -f ".$self->{TESTDIR}."/tmp/cfg.file -m ".$self->{MODULES_PATH});
+	if (defined $ENV{"TRACEME"} && $ENV{"TRACEME"} eq 'strace') {
+		system("strace -tt -s 512 -o strace ".$self->{LIGHTTPD_PATH}." -D -f ".$self->{SRCDIR}."/".$self->{CONFIGFILE}." -m ".$self->{MODULES_PATH}." &");
+	} elsif (defined $ENV{"TRACEME"} && $ENV{"TRACEME"} eq 'valgrind') {
+		system("valgrind --tool=memcheck --show-reachable=yes --leak-check=yes --log-file=valgrind ".$self->{LIGHTTPD_PATH}." -D -f ".$self->{SRCDIR}."/".$self->{CONFIGFILE}." -m ".$self->{MODULES_PATH}." &");
 	} else {
-		system("valgrind --tool=memcheck --show-reachable=yes --leak-check=yes --logfile=foo ".$self->{LIGHTTPD_PATH}." -D -f ".$self->{TESTDIR}."/tmp/cfg.file -m ".$self->{MODULES_PATH}." &");
+		system($self->{LIGHTTPD_PATH}." -f ".$self->{SRCDIR}."/".$self->{CONFIGFILE}." -m ".$self->{MODULES_PATH});
 	}
 
 	select(undef, undef, undef, 0.1);
@@ -196,6 +197,9 @@ sub handle_http {
 			}
 		}
 
+		$t->{etag} = $resp_hdr{'etag'};
+		$t->{date} = $resp_hdr{'date'};
+
 		# check length
 		if (defined $resp_hdr{"content-length"}) {
 			$resp_body = substr($lines, 0, $resp_hdr{"content-length"});
