@@ -1,25 +1,30 @@
+#include "config.h"
+#ifdef HAVE_FASTCGI_FASTCGI_H
+#include <fastcgi/fcgi_stdio.h>
+#else
 #include <fcgi_stdio.h>
+#endif
 #include <stdlib.h>
 #include <unistd.h>
 #include <string.h>
 
 int main () {
 	char* p;
-	
-	while (FCGI_Accept() >= 0) {   
+
+	while (FCGI_Accept() >= 0) {
 		/* wait for fastcgi authorizer request */
-		
+
 		printf("Content-type: text/html\r\n");
-		
+
 		if (((p = getenv("QUERY_STRING")) == NULL) ||
 		    strcmp(p, "ok") != 0) {
 			printf("Status: 403 Forbidden\r\n\r\n");
-		} else { 
+		} else {
 			printf("\r\n");
 			/* default Status is 200 - allow access */
 		}
-		
-		printf("foobar\r\n");  
+
+		printf("foobar\r\n");
 	}
 
 	return 0;
