@@ -1,15 +1,20 @@
+#include "config.h"
+#ifdef HAVE_FASTCGI_FASTCGI_H
+#include <fastcgi/fcgi_stdio.h>
+#else
 #include <fcgi_stdio.h>
+#endif
 #include <stdlib.h>
 #include <unistd.h>
 #include <string.h>
 
 int main () {
 	int num_requests = 2;
-	
+
 	while (num_requests > 0 &&
 	       FCGI_Accept() >= 0) {
 		char* p;
-		
+
 		if (NULL != (p = getenv("QUERY_STRING"))) {
 			if (0 == strcmp(p, "lf")) {
 				printf("Status: 200 OK\n\n");
@@ -32,9 +37,9 @@ int main () {
 		} else {
 			printf("Status: 500 Internal Foo\r\n\r\n");
 		}
-		 
+
 		printf("test123");
 	}
-	
+
 	return 0;
 }
